=begin

    Copyright The Resistance (C) 1995-2011

    Author: blass
    blass[at]sdf.lonestar.org

    Badass 0.1 alpha

    This file is part of Badass.

    Badass is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Badass is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Badass.  If not, see <http://www.gnu.org/licenses/>.

=end

module Net # :nodoc:
  module DNS

    module HashKeys # :nodoc:

      # Returns an hash with all the
      # keys turned into downcase
      #
      #   hsh = {"Test" => 1, "FooBar" => 2}
      #   hsh.downcase_keys!
      #      #=> {"test"=>1,"foobar"=>2}
      #
      def downcase_keys!
        hsh = Hash.new
        self.each do |key,val|
          hsh[key.downcase] = val
        end
        self.replace(hsh)
      end
      
    end

    module HashOperators # :nodoc:

      # Performs a sort of group difference
      # operation on hashes or arrays
      #
      #   a = {:a=>1,:b=>2,:c=>3}
      #   b = {:a=>1,:b=>2}
      #   c = [:a,:c]
      #   a-b #=> {:c=>3}
      #   a-c #=> {:b=>2}
      #
      def -(other)
        case other
          when Hash
            delete_if { |k,v| other.has_key?(k) }
          when Array
            delete_if { |k,v| other.include?(k) }
        end
      end

    end

  end
end


class Hash # :nodoc:
  include Net::DNS::HashKeys
  include Net::DNS::HashOperators
end
