=begin

    Copyright The Resistance (C) 1995-2011

    Author: blass
    blass[at]sdf.lonestar.org

    Badass 0.1 alpha

    This file is part of Badass.

    Badass is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Badass is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Badass.  If not, see <http://www.gnu.org/licenses/>.

=end

module Racket
  module Misc
    # Simple class to represent a datastructure that is made up of a 
    # null terminted string followed by an arbitrary number of
    # arbitrarily sized values, followed by a "rest" field.
    class VT 
      # the value for this VT object
      attr_accessor :value
      # the array of types for this VT object
      attr_accessor :types
      # everything else
      attr_accessor :rest
      
      # Create a new VT which consists of a null terminated string
      # followed by some number of arbitrarily sized values, as 
      # specified by +args+
      def initialize(*args)
        @lengths = args
        @types = []
        @value = ""
      end 
      
      # Given +data+, return the value and an array
      # of the types as dictated by this instance
      def decode(data)
        null = data.index(0x00)
        value = data.unpack("a#{null}")[0]
        data = data.slice(null+1, data.length)
        
        n = 0
        types = []
        @lengths.each do |l|
          types[n] = data.unpack("#{punpack_string(l)}")[0]
          data = data.slice(l, data.length)
          n += 1
        end
        
        [value, types, data]
      end
      
      # Given +data+, set the +value+ and +types+ array
      # accordingly
      def decode!(data)
        @value, @types, @rest = self.decode(data)
      end
      
      # Return a string suitable for use elsewhere
      def encode
        s = "#{@value}\000"
        
        n = 0
        @lengths.each do |l|
          s << [@types[n]].pack("#{punpack_string(l)}")
          n += 1
        end
        s
      end
      
      def to_s
        encode
      end
      
      def to_str
        encode
      end
      
      private
      # XXX: make this handle arbitrarily sized fields
      def punpack_string(size)
        s = ""
        case size
        when 1
          s << "C"
        when 2
          s << "n"
        when 4
          s << "N"
        else
          raise ArgumentError, "Size #{s} not supported"
        end
        s
      end
      
    end
  end
end
# vim: set ts=2 et sw=2:
